-- Puffin: A Tojaso FowlPlay Project
-- Adapter component for assembling castbars and updating them as needed.

local Adapter_Castbars = {}
local EF = Puffin.ErrorFormat
local DB, CB
local active_castbars = {}
local castbar_changed = {}
local configuration_changed = true

local function CastbarChanged(event, unit) castbar_changed[unit] = true end

function Adapter_Castbars.OnInitialize()	
	Puffin.RegisterCallback("Casting", CastbarChanged)
end

function Adapter_Castbars.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, EF("Adapter_Castbars", "Data component not found"))
	CB = Puffin.Component("View_CastBar")
	assert(CB, EF("Adapter_Castbars", "View_CastBar component not found"))
end

-- On first update, configure the brokers, afterwards just update each castbar as needed
function Adapter_Castbars.OnUpdate()
	local layout = DB.GetRecord("layouts", "castbars")
	if not layout or not layout.castbars then return end
	if configuration_changed then
		for unit, cb in pairs(layout.castbars) do -- only configure castbars once, they will never be released
			local u = CB.Create() -- create the castbar view
			local v = layout -- castbar layout containing widget descriptors
			u.defaults = v.defaults; u.depth = v.depth; u.layout = v; u.scale = v.scale or 1; u.alpha = v.alpha or 1; u.hidden = true; u._unit = unit
			u.offsetX = cb.x or 0; u.offsetY = cb.y or 0; u.anchorPoint = cb.anchorPoint or "BOTTOMLEFT"; u.relativePoint = cb.relativePoint or "BOTTOMLEFT"
			u.width = cb.width; u.height = cb.height; u.iconSize = cb.iconSize; u.iconSide = cb.iconSide or "left"; u.iconOffset = cb.iconOffset or 0
			u.labelSize = cb.labelSize; u.timerSize = cb.timerSize
			CB.Configure(u)
			active_castbars[unit] = u
			Puffin.Callback("CastingUnitTracking", unit, true)
		end
		configuration_changed = false -- all created castbars have been processed
	end

	for unit, isChanged in pairs(castbar_changed) do -- update all the changed castbars
		if isChanged then
			local cb = DB.GetRecord("casting", unit)
			local u = active_castbars[unit]
			u.hidden = not cb
			if u then CB.Update(u, cb) end
		end
		castbar_changed[unit] = false
	end
end

Puffin.RegisterComponent("Adapter_Castbars", Adapter_Castbars, "ADAPTER", 2)
