-- Puffin: A Tojaso FowlPlay Project
-- Component containing anchor management functions allowing views to be positioned relative to other views.

-- Exported functions:
-- RegisterView() - register a view to either serve as a dynamic anchor or to position relative to a registered view
-- UnregisterView() - unregister a previously registered view

local View_AnchorManagement = {}
local EF = Puffin.ErrorFormat
local DB
local views = {} -- registered views

-- Register a view to either serve as a dynamic anchor or to position relative to a registered view
function View_AnchorManagement.RegisterView(name, view)
	assert(view and view.layout, EF("View_AnchorManagement", "view without layout"))
	views[name] = view
end

-- Unregister a previously registered view
function View_AnchorManagement.UnregisterView(name) views[name] = nil end

-- Update positions of all registered views that depend on dynamic anchors
function View_AnchorManagement.Update()
	for name, v in pairs(views) do
		if v.relativeTo then
			local rbg = views[v.relativeTo]
			if rbg then
				if rbg.count == 0 then
					local i = 0
					while (rbg.count == 0) and rbg.relativeEmpty and rbg.relativeTo do
						rbg = views[rbg.relativeTo]
						i = i + 1; if i > 20 then break end -- safety check, never loop more than 20 deep
					end
				end
				local align, offsetX, offsetY = "BOTTOMLEFT", 0, 0
				if (rbg.count > 0) or not v.relativeEmpty then
					offsetX, offsetY = v.relativeX / v.scale, v.relativeY / v.scale
					if v.relativeLastBar then -- alternative is to attach to the last bar rendered back in the other bar group
						align = rbg.anchorPoint
						if rbg.lastX and rbg.lastY then offsetX = offsetX + rbg.lastX; offsetY = offsetY + rbg.lastY
						elseif v.relativeRow and rbg.lastRow then offsetX = offsetX + rbg.lastRow
						elseif v.relativeColumn and rbg.lastColumn then offsetY = offsetY + rbg.lastColumn end
					end
				end
				v.frame:ClearAllPoints(); v.frame:SetPoint(align, rbg.frame, align, offsetX, offsetY)
				--[[	
				print("relative", name, v.count, rbg.name, rbg.count, align, offsetX, offsetY)
				local tc = v.frame; tc.tex = tc:CreateTexture(nil, "ARTWORK"); tc.tex:SetSize(10, 10); tc.tex:SetPoint("CENTER", tc, "CENTER");
					tc.tex:SetTexture(0,0,1,0.5); tc.tex:Show() -- debug
				local tc = rbg.frame; tc.tex = tc:CreateTexture(nil, "ARTWORK"); tc.tex:SetSize(10, 10); tc.tex:SetPoint("CENTER", tc, "CENTER");
					tc.tex:SetTexture(0,1,0,0.5); tc.tex:Show() -- debug
				]]--
			end
		end
	end
end

function View_AnchorManagement.OnEnable()
	DB = Puffin.Component("Data") -- link to data management functions
	assert(DB, EF("View_AnchorManagement", "Data component not found"))
end

Puffin.RegisterComponent("View_AnchorManagement", View_AnchorManagement, "VIEW", 1)
